import os
from StringIO import StringIO
from OFS.ObjectManager import BadRequestException
from Products.CMFCore import permissions
from Products.ZCatalog.ZCatalog import manage_addZCatalog
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import listTypes
from Products.Archetypes.config import REFERENCE_CATALOG
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.CMFPlacefulWorkflow.PlacefulWorkflowTool import WorkflowPolicyConfig_id
from Products.CMFEditions.Permissions import RevertToPreviousVersions

from Products.Intranet import config
from Products.Intranet.utils import installDepends
from Products.Intranet.utils import add_form_controller_overrides
from Products.Intranet.utils import remove_form_controller_overrides

from zope.interface import directlyProvides, directlyProvidedBy
#from Products.Intranet.Extensions.utils import setupKupu

def fixUpEditTab(portal, out):
    pt=getToolByName(portal, 'portal_types')
    action=pt.Document._actions[1]
    action.permissions=(permissions.View,)
    print >> out, "edit tabs set to view permission"

def navTreeSetup(portal, out):
    nav_types = config.NAVTREE_TYPES
    portal.prefs_navigation_set(generated_tabs=False, portaltypes=nav_types)
    print >> out, '-> Navigation Tree types set'

def hideActions(portal, out):
    actions_map = {#'portal_teams': ('user/myteamspaces',),
                   'portal_actions': ('folder_buttons/change_state',
                                      'document_actions/full_screen',),
                   'portal_undo': ('user/undo',)
                   }
    for tool_id in actions_map.keys():
        tool = getToolByName(portal, tool_id)
        for action_id in actions_map[tool_id]:
            action = tool.getActionObject('%s' % action_id)
            if action is not None and action.visible:
                print >> out, "-> hiding '%s' action" % action_id
                action.edit(visible=False)

def hideActionTabs(portal, out):
    """
    we need to iterate through all of the portal types to hide the
    certain tabs... *sigh*
    """
    ttool = getToolByName(portal, 'portal_types')
    ftis = ttool.listTypeInfo()
    hidden_actions = ('object/local_roles', 'object/metadata')
    for fti in ftis:
        for act_id in hidden_actions:
            print >> out, "Hiding '%s' action for all types" % act_id
            action = fti.getActionObject(act_id)
            if action is not None:
                action.edit(visible=False)

def setupPortalActions(portal, out):
    print >> out, '-> Changing condition on the "home" portal tab'
    atool = getToolByName(portal, 'portal_actions')
    action = atool.getActionObject('portal_tabs/index_html')
    #action.edit(title="Intranet Home",
    #                 visible=False)

    mtool = getToolByName(portal, 'portal_membership')
    #action = mtool.getActionObject('user/mystuff')
    #action.edit(title='My Home')

    action = mtool.getActionObject('user/preferences')
    action.edit(title='My Info',
                action="string:${portal_url}/portal_memberdata/${member/id}/edit")

    print >> out, '-> Adding "forum" portal tab'
    if atool.getActionObject('portal_tabs/forum') is None:
        atool.addAction('forum', 'Forumas',
                    'string:$portal_url/forum',
                    '',
                    'View', 'portal_tabs', True)

    print >> out, '-> Adding "files" portal tab'
    if atool.getActionObject('portal_tabs/files') is None:
        atool.addAction('files', 'Failai',
                    'string:$portal_url/files',
                    '',
                    'View', 'portal_tabs', True)

    print >> out, '-> Adding "gallery" portal tab'
    if atool.getActionObject('portal_tabs/gallery') is None:
        atool.addAction('gallery', 'Galerija',
                    'string:$portal_url/gallery',
                    '',
                    'View', 'portal_tabs', True)

    print >> out, '-> Adding "wiki" portal tab'
    if atool.getActionObject('portal_tabs/wiki') is None:
        atool.addAction('gallery', 'Wiki',
                    'string:$portal_url/wiki',
                    '',
                    'View', 'portal_tabs', True)

    print >> out, '-> Adding "events" portal tab'
    if atool.getActionObject('portal_tabs/events') is None:
        atool.addAction('gallery', 'Ivykiai',
                    'string:$portal_url/events',
                    '',
                    'View', 'portal_tabs', True)

def uiTweaks(portal, out):
    print >> out, 'Applying site UI customizations'

    #setupSitePortlets(portal, out)

    print >> out, '-> Removing not addable types'
    ttool = getToolByName(portal, 'portal_types')
    for type in config.NOT_ADDABLE_TYPES:
        fti = ttool.getTypeInfo(type)
        if fti is not None:
            fti.global_allow = False

    #hideActions(portal, out)
    #navTreeSetup(portal, out)
    #setupPortalActions(portal, out)
    #hidePlonePortlets(portal, out)
    #hideActionTabs(portal, out)
    #addProjectConfigAction(portal, out)
    #addUndeleteAction(portal, out)

def addForum(portal, out):
    # Main Forum added there
    if not 'forum' in portal.objectIds():
        print >> out, 'Creating Intranet Forum'
        forum = getToolByName(portal, 'portal_types')
        forum.constructContent('Ploneboard', portal, 
                               'forum', title='Forumas', categories='Bendras')
                               
        # Some default forums
        folder = portal._getOb('forum')
        folder.invokeFactory('PloneboardForum', 'Renginiai', title = 'Renginiai', categories='Bendras')

def addWiki(portal, out):
    # Wiki folder add and configured here.
    folder_id = 'wiki'
    if folder_id not in portal.objectIds():
        print >> out, 'Creating Intranet Wiki'
        portal.invokeFactory('Folder', folder_id, title = "Wiki", description = "Intraneto Wiki")
        folder = portal._getOb(folder_id)
        
        # Add type restrictions
        print >> out, 'Restricting addable types in Wiki Folder'
        folder.setConstrainTypesMode(1)
        folder.setLocallyAllowedTypes(["IronicWiki"])

        # Add first Wiki page
        topic_id = 'pradinis'
        default_text = 'Jūs galite pridėti ((naują puslapį)), kurio pavadinimas apskliaustas dvigubais skliaustais. Tai bus nuoroda į naujai sukurtą puslapį. Paspaudus ant šios nuorodos bus sukurtas naujas puslapis, kurį iškart galite redaguoti. Jei jau puslapis tokiu pavadinimu egzistuoja, atsiras nuoroda į tą puslapį'
        folder.invokeFactory('IronicWiki', topic_id, title = "Pradinis Wiki puslapis", body = default_text)
        folder.setDefaultPage(topic_id)

def addGallery(portal, out):
    # Adding Intranet photo Gallery.
    folder_id = 'gallery'
    if folder_id not in portal.objectIds():
        print >> out, 'Creating Intranet Gallery'
        portal.invokeFactory('ImageRepository', folder_id, title = 'Gelerija', description = 'Intraneto nuotraukų galerija')

def addFiles(portal, out):
    # Adding File gallery.
    folder_id = 'files'
    if folder_id not in portal.objectIds():
        print >> out, 'Creating Intranet file gallery'
        portal.invokeFactory('Folder', folder_id, title = "Failai", description = "Failų saugykla")
        folder = portal._getOb(folder_id)
        
        # Add type restrictions
        print >> out, 'Restricting addable types in Failai Folder'
        folder.setConstrainTypesMode(1)
        folder.setLocallyAllowedTypes(["File"])

def registerCSS(portal, out):
    print >> out, 'Registering style sheets'
    cssreg = getToolByName(portal, 'portal_css')
    cssreg.registerStylesheet('intranet_style.css')
    '''cssreg.registerStylesheet('ts.css')
    cssreg.registerStylesheet('ts_portlets.css')
    cssreg.registerStylesheet('openplans_print.css',
                              media="print")'''

def setCategories(portal, out):
    """Setup project categories
    """
    print >> out, '-> creating initial project categories'
    ttool = getToolByName(portal, 'portal_types')
    ctool = getToolByName(portal, 'portal_projectmap')
    utils = getToolByName(portal, 'plone_utils')
    def createCategories(root, catdefs, special=False):
        for cat in catdefs:
            newid = utils.normalizeString(cat['title'])
            catspecial = cat.get('special', special)
            at_type = catspecial and 'SpecialProjectCategory' or 'ProjectCategory'
            catundef = cat.get('undefined', False)
            if catundef:
                at_type = 'UndefinedCategory'
            ttool.constructContent(at_type, root, newid, title=cat['title'])

            created = root._getOb(newid)
            if cat.has_key('children'):
                createCategories(created, cat['children'], catspecial)

    createCategories(ctool, config.INITIAL_CATEGORIES)
    
def setupVersioning(portal, out):
    versioned_types = ('IronicWiki')
    ttool = getToolByName(portal, 'portal_types')
    version_tool = getToolByName(portal, 'portal_repository', None)
    diff_tool = getToolByName(portal, 'portal_diff', None)
    if version_tool is not None:
        version_tool.setVersionableContentTypes(versioned_types)
    for p_type in versioned_types:
        if version_tool is not None:
            version_tool.addPolicyForContentType(p_type,
                                                    'at_edit_autoversion')
        if diff_tool is not None:
            diff_tool.manage_addDiffField(p_type, 'none',
                                             'Compound Diff for AT types')
        print >> out, 'Enabled versioning for %s'%p_type
    if diff_tool is not None:
        ttool.ChangeSet.global_allow = False

def changeFrontPage(portal, out):
    # Change front-page text to Lithuanian.
    front_page_text = '<p><b>Mokyklos intranetas</b> – vidinis mokyklos kompiuterių tinklas, kuriame informacija naudojama ir tvarkoma interneto technologijų priemonėmis. Mokyklos intranetas gali turėti ryšį su internetu. Jis skirtas mokyklos bendruomenei.</p>'
    page = portal._getOb('front-page')
    page.setTitle('Sveiki atvykę į mokyklos intranetą')
    page.setText(front_page_text)

def renameActions(portal, out):
    # Change event, news, members to lithuanian names
    #portal.manage_renameObject('events', 'Ivykiai')
    #portal.manage_renameObject('Members', 'Nariai')
    #portal.manage_renameObject('news', 'Naujienos')
    folder = portal._getOb('events')
    folder.setTitle('Įvykiai')
    folder = portal._getOb('news')
    folder.setTitle('Naujienos')
    folder.setDescription('')
    
    """set up the 'nariai' folder"""
    oldname = 'Members'
    newname = 'nariai'
    newtitle = 'Nariai'
    print >> out, 'Setting up "%s" folder' % newname
    obj_ids = portal.objectIds()
    if oldname in obj_ids:
        print >> out, '-> Renaming "%s" -> "%s"' % (oldname, newname)
        folder = portal._getOb(oldname)
        fti = folder.getTypeInfo()
        changeback = False
        if not fti.global_allow:
            oldvalue = fti.global_allow
            changeback = True
            fti.global_allow = True
        portal.manage_renameObject(oldname, newname)
        folder.setTitle(newtitle)
        if changeback:
            fti.global_allow = oldvalue
    elif not newname in obj_ids:
        print >> out, '-> Creating "%s" folder' % newname
        ttool = getToolByName(portal, 'portal_types')
        ttool.constructContent('Large Plone Folder', portal,
                               newname, title=newtitle)

def install(self, migrate_atdoc_to_wickeddoc=True):
    out = StringIO()
    portal = getToolByName(self, 'portal_url').getPortalObject()
    installDepends(self)
    install_subskin(self, out, config.GLOBALS)
    installTypes(portal, out, listTypes(config.PROJECTNAME), config.PROJECTNAME)
    uiTweaks(portal, out)
    addForum(portal, out)
    addWiki(portal, out)
    addGallery(portal, out)
    addFiles(portal, out)
    changeFrontPage(portal, out)
    renameActions(portal, out)
    registerCSS(portal, out)
    #setupKupu(portal, out)
    #installColumns(portal, out)
    #if migrate_atdoc_to_wickeddoc: # needed for unit test sanity
    #    migrateATDocToWickedDoc(portal, out)
    setupVersioning(portal, out)
    #fixUpEditTab(portal, out)
    #setSiteIndexPage(portal, out)
    #setCategories(portal, out)

    print >> out, "Successfully installed %s." % config.PROJECTNAME
    
    # create_test_content(self)
    
    return out.getvalue()
