"""
for setup widgets for those annoying little tasks that
only need to happen occasionally
"""
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.setup.SetupBase import SetupWidget
from Products.CMFPlone import MigrationTool
from Products.Archetypes.public import listTypes
from Products.Archetypes.Extensions.utils import installTypes
from Products.Intranet import config
from Products.Intranet.workflows import PLACEFUL_POLICIES
from zLOG import INFO, ERROR

from utils import setupKupu
from Install import installColumns, fixUpEditTab, hideActions, \
     installWorkflows, setupPortalActions, addFormControllerOverrides, \
     installWorkflowPolicies
from cStringIO import StringIO

try:
    set = set
except NameError:
    from sets import Set as set

from Products.Intranet.config import PROJECTNAME

def setupProjectLayout(self, portal):
    portal.projects.setLayout('project_listing')

out = StringIO()
def convertFunc(func):
    """
    turns a standard install function into a
    setup widget function
    """
    def new_func(self, portal):
        func(portal, out)
    #blank out
    out=StringIO()
    return new_func

def reinstallWorkflows(self, portal):
    wftool = getToolByName(portal, 'portal_workflow')
    qi = getToolByName(portal, 'portal_quickinstaller')
    product = getattr(qi, PROJECTNAME)
    wfs = product.getWorkflows()
    wftool.manage_delObjects(ids=wfs)
    out = StringIO()
    installWorkflows(portal, out)

def reinstallWorkflowPolicies(self, portal):
    pwftool = getToolByName(portal, 'portal_placeful_workflow')
    policies = set(pwftool.objectIds())
    deletes = policies.intersection(set(PLACEFUL_POLICIES.keys()))
    pwftool.manage_delObjects(ids=list(deletes))
    out = StringIO()
    installWorkflowPolicies(portal, out)

def reinstallTypes(self, portal):
    out = StringIO()
    installTypes(portal, out, listTypes(config.PROJECTNAME),
                 config.PROJECTNAME)

functions = dict(
    setupKupu = convertFunc(setupKupu),
    fixUpEditTab = convertFunc(fixUpEditTab),
    installMetadataColumns = convertFunc(installColumns),
    setProjectListingLayout = setupProjectLayout,
    setupPortalActions = convertFunc(setupPortalActions),
    hideActions = convertFunc(hideActions),
    reinstallWorkflows = reinstallWorkflows,
    reinstallWorkflowPolicies = reinstallWorkflowPolicies,
    reinstallTypes = reinstallTypes,
    addFormControllerOverrides = convertFunc(addFormControllerOverrides),
    )

class TOPPSetup(SetupWidget):
    """ Intranet Setup Bucket Brigade  """

    type = 'TOPP Setup'

    description = ' utillity methods for TOPP site setup '

    def setup(self):
        pass

    def run(self, fn, **kwargs):
        out = []
        out.append((functions[fn](self, self.portal, **kwargs),INFO))
        out.append(('Function %s has been applied' % fn, INFO))
        return out

    def addItems(self, fns):
        out = []
        for fn in fns:
            out.append((functions[fn](self, self.portal),INFO))
            out.append(('Function %s has been applied' % fn, INFO))
        return out

    def installed(self):
        return []

    def available(self):
        """ Go get the functions """
        return functions.keys()
    
MigrationTool.registerSetupWidget(TOPPSetup)
